package com.zsc.edu.dify.modules.dify.controller;

import com.zsc.edu.dify.exception.ApiException;
import com.zsc.edu.dify.exception.ExceptionUtil;
import com.zsc.edu.dify.framework.security.SecurityUtil;
import com.zsc.edu.dify.modules.dify.service.AppEntityService;
import com.zsc.edu.dify.modules.operationLog.entity.OperationLogAnnotation;
import io.github.guoshiqiufeng.dify.chat.DifyChat;
import io.github.guoshiqiufeng.dify.chat.dto.request.ChatMessageSendRequest;
import io.github.guoshiqiufeng.dify.chat.dto.request.MessageConversationsRequest;
import io.github.guoshiqiufeng.dify.chat.dto.request.MessagesRequest;
import io.github.guoshiqiufeng.dify.chat.dto.response.AppParametersResponseVO;
import io.github.guoshiqiufeng.dify.chat.dto.response.ChatMessageSendCompletionResponse;
import io.github.guoshiqiufeng.dify.chat.dto.response.ChatMessageSendResponse;
import io.github.guoshiqiufeng.dify.chat.dto.response.MessageConversationsResponse;
import io.github.guoshiqiufeng.dify.core.pojo.DifyPageResult;
import io.github.guoshiqiufeng.dify.core.pojo.response.MessagesResponseVO;
import jakarta.annotation.Resource;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;

import java.util.List;

/**
 * @author yanghq
 * @version 1.0
 * @since 2025/3/25 10:48
 */
@RestController
@RequestMapping("/api/v1/chat")
public class V1ChatController {

    @Resource
    private DifyChat difyChat;
    @Resource
    private AppEntityService appEntityService;

    /**
     * 发送消息
     *
     * @param sendRequest 消息参数 （可以自定义参数，调用 difyChat 实例时重新组装即可），
     *                    用户 id可以改为从上下文（token）获取，
     *                    apikey 建议在数据库进行存储，前端调用时传智能体 id，从数据库查询
     */
    @PostMapping("/completions/{appId}")
    @OperationLogAnnotation(content = "'dify对话'", operationType = "发送")
    public ChatMessageSendResponse sendChatMessage(
            @RequestBody ChatMessageSendRequest sendRequest,
            @PathVariable String appId
    ) {
        sendRequest.setApiKey(appEntityService.getApikey(appId));
        sendRequest.setUserId(SecurityUtil.getUserInfo().id.toString());
        return ExceptionUtil.difyException(()->difyChat.send(sendRequest));
    }


    /**
     * 发送消息（流式）
     *
     * @param sendRequest 消息参数 （可以自定义参数，调用 difyChat 实例时重新组装即可），
     *                    用户 id可以改为从上下文（token）获取，
     *                    apikey 建议在数据库进行存储，前端调用时传智能体 id，从数据库查询
     */
    @PostMapping(value = "/completions/stream/{appid}", produces = MediaType.TEXT_EVENT_STREAM_VALUE)
//    @PreAuthorize("hasAuthority('difyChat:query')")
    public Flux<ChatMessageSendCompletionResponse> sendChatMessageStream(
            @RequestBody ChatMessageSendRequest sendRequest,
            @PathVariable String appid
    ) {
        sendRequest.setApiKey(appEntityService.getApikey(appid));
        return difyChat.sendChatMessageStream(sendRequest);
    }

    /**
     * 获取会话列表
     *
     * @param request 请求参数
     * @return 会话列表
     */
    @PostMapping("/conversations/{appId}")
    public DifyPageResult<MessageConversationsResponse> conversations(
            @RequestBody MessageConversationsRequest request,
            @PathVariable String appId
    ) {
        request.setApiKey(appEntityService.getApikey(appId));
        request.setUserId(SecurityUtil.getUserInfo().id.toString());
        return difyChat.conversations(request);
    }

    /**
     * 获取消息列表
     *
     * @param request 请求参数
     * @return 消息列表
     */
    @PostMapping("/messages/{appid}")
    public DifyPageResult<MessagesResponseVO> messages(
            @RequestBody MessagesRequest request,
            @PathVariable String appid
    ) {
        String apiKey = appEntityService.getApikey(appid);
        request.setUserId(SecurityUtil.getUserInfo().id.toString());
        request.setApiKey(apiKey);
        return ExceptionUtil.difyException(()->difyChat.messages(request));
    }

    /**
     * 停止流式消息
     *
     * @param taskId  任务id
     */
    @PatchMapping("/stream/stop")
    public void stopMessagesStream(@RequestParam String taskId, @RequestParam String appId) {
        String apiKey = appEntityService.getApikey(appId);
        String userId = SecurityUtil.getUserInfo().id.toString();
        difyChat.stopMessagesStream(apiKey, taskId,userId);
    }

    /**
     * 删除会话
     *
     * @param conversationId 会话id
     */
    @DeleteMapping("/conversation")
    @OperationLogAnnotation(content = "'dify对话'", operationType = "删除")
    public void deleteConversation(@RequestParam String conversationId, @RequestParam String appId) {
        String apiKey = appEntityService.getApikey(appId);
        String userId = SecurityUtil.getUserInfo().id.toString();
        try{
            difyChat.deleteConversation(conversationId, apiKey,userId);
        }catch (RuntimeException e){
            throw new ApiException("删除会话失败"+e.getMessage());
        }
    }

    /**
     * 获取会话建议
     * @param messageId
     * @param appId
     * @return
     */
    @GetMapping("/messages/suggested")
    public List<String> messagesSuggested(String messageId,String appId){
        String apiKey = appEntityService.getApikey(appId);
        String userId = SecurityUtil.getUserInfo().id.toString();
        return ExceptionUtil.difyException(()->difyChat.messagesSuggested(messageId,apiKey,userId));
    }

    /**
     * 获取应用参数
     * @param appid
     * @return
     */
    @GetMapping("/parameters/{appid}")
    public AppParametersResponseVO parameters(@PathVariable String appid){
        String apiKey = appEntityService.getApikey(appid);
        return ExceptionUtil.difyException(()->difyChat.parameters(apiKey));
    }

}